
-- Written by Ian Lynagh <igloo@debian.org> in 2003.
-- Released into the public domain.

module Main (main) where

import Graphics.Rendering.OpenGL.GL
import Graphics.Rendering.OpenGL.GL.Capability
import Graphics.UI.GLUT as GLUT
import System(ExitCode(..), exitWith)
import Data.IORef

type St = (GLfloat, GLfloat, GLfloat)

time :: Int
time = 1

dx, dy, dz :: GLfloat
dx = 0.1
dy = 0.3
dz = 0.7

red, green, yellow, blue, purple, cyan :: Color3 GLfloat
red    = Color3 1 0 0
green  = Color3 0 1 0
yellow = Color3 1 1 0
blue   = Color3 0 0 1
purple = Color3 1 0 1
cyan   = Color3 0 1 1

main :: IO ()
main = do getArgsAndInitialize
          initialDisplayMode $= [ WithDepthBuffer, DoubleBuffered, RGBAMode ]
          win <- createWindow "Hello World"
          myInit
          makeCapability CapDepthTest $= Enabled
          drawBuffer $= BackBuffers
          ref <- newIORef (0, 0, 0)
          displayCallback $= (display ref)
          keyboardMouseCallback $= Just keyboardMouse
          addTimerCallback time (inc_anim ref)
          mainLoop

inc_anim :: IORef St -> IO ()
inc_anim x = do (r_x, r_y, r_z) <- readIORef x
                writeIORef x (r_x + dx, r_y + dy, r_z + dz)
                postRedisplay Nothing
                addTimerCallback time (inc_anim x)

myInit :: IO () 
myInit = do clearColor $= (Color4 0.0 0.0 0.0 0.0)
            matrixMode $= Projection
            loadIdentity
            ortho 0.0 1.0 0.0 1.0 (-1.0) 1.0

keyboardMouse :: KeyboardMouseCallback
keyboardMouse (Char '\27') _ _ _ = exitWith ExitSuccess
keyboardMouse (Char 'q')   _ _ _ = exitWith ExitSuccess
keyboardMouse _ _ _ _ = return ()

display :: IORef St -> DisplayCallback
display x = do draw_colourful_cube x
               swapBuffers

draw_colourful_cube :: IORef St -> IO ()
draw_colourful_cube x =
               do (r_x, r_y, r_z) <- readIORef x
                  clear [DepthBuffer, ColorBuffer]
                  loadIdentity
                  rotate r_x (Vector3 1 0 0 :: Vector3 GLfloat)
                  rotate r_y (Vector3 0 1 0 :: Vector3 GLfloat)
                  rotate r_z (Vector3 0 0 1 :: Vector3 GLfloat)
                  mapM_ draw_face (zip colours faces)
    where draw_face :: (Color3 GLfloat, IO ()) -> IO ()
          draw_face (colour, face) = do color colour
                                        renderPrimitive Quads face
          faces = map (mapM_ vertex) face_vertices :: [IO ()]
          colours = [red, green, yellow, blue, purple, cyan]
          face_vertices = [
                 [Vertex3     to     to     to,
                  Vertex3   from     to     to,
                  Vertex3   from   from     to,
                  Vertex3     to   from     to],
                 [Vertex3     to     to   from,
                  Vertex3   from     to   from,
                  Vertex3   from   from   from,
                  Vertex3     to   from   from],
                 [Vertex3     to     to     to,
                  Vertex3   from     to     to,
                  Vertex3   from     to   from,
                  Vertex3     to     to   from],
                 [Vertex3     to   from     to,
                  Vertex3   from   from     to,
                  Vertex3   from   from   from,
                  Vertex3     to   from   from],
                 [Vertex3     to     to     to,
                  Vertex3     to   from     to,
                  Vertex3     to   from   from,
                  Vertex3     to     to   from],
                 [Vertex3   from     to     to,
                  Vertex3   from   from     to,
                  Vertex3   from   from   from,
                  Vertex3   from     to   from]]

from, to :: GLfloat
from = -0.4
to   =  0.4

